package de.fhdw.wtf.parser.test;

import static org.junit.Assert.assertEquals;

import java.util.List;
import java.util.Vector;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.ConstructorByTypeAndSignatureState;
import de.fhdw.wtf.common.ast.ConstructorReference;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.OperationModifier;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.dsl.scanner.test.VerboseTokenStream;
import de.fhdw.wtf.parser.Parser;

/**
 * Tests the parsing of constructors.
 */
public class TestConstructors {
	/**
	 * Parser which is used to parse the {@link VerboseTokenStream} from scanner.
	 */
	private Parser parser;
	/**
	 * The {@link CurlyBracketOpenToken} represents "{".
	 */
	private CurlyBracketOpenToken curlyBracketOpenToken;
	
	/**
	 * The {@link BracketOpenToken} represents "(".
	 */
	private BracketOpenToken bracketOpenToken;
	
	/**
	 * The {@link BracketCloseToken} represents ")".
	 */
	// private BracketCloseToken bracketCloseToken;
	
	/**
	 * The String token, is an {@link IdentifierToken}, that represents "String".
	 */
	private IdentifierToken stringIdentifier;
	/**
	 * The Integer token, is an {@link IdentifierToken}, that represents "Integer".
	 */
	private IdentifierToken integerIdentifier;
	
	/**
	 * Creates the most important Tokens.
	 */
	@Before
	public void setUp() {
		this.bracketOpenToken = BracketOpenToken.create(Position.create("", 1, 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 1, 1, 0));
		this.stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		
	}
	
	/**
	 * Group1:group=[A:class={};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testClassWithoutConstructor() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = (IdentifierToken) output.getStream().peek();
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 15, 6));
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, group1IdentifierToken);
		
		final Name aName = groupName.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(aClass);
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testSimpleConstructorWithoutParameters() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = (IdentifierToken) output.getStream().peek();
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 15, 6));
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, group1IdentifierToken);
		
		final Name aName = groupName.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(aClass);
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		
		constructors.add(Constructor.create(product, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[A:class={();};B:class={();};C:class={();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testManyClassesWithOneConstructorOneGroup() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={();};B:class={();};C:class={();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = (IdentifierToken) output.getStream().peek();
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 15, 6));
		final IdentifierToken bIdentifierToken = IdentifierToken.create("B", Position.create("", 1, 29, 16));
		final IdentifierToken cIdentifierToken = IdentifierToken.create("C", Position.create("", 1, 43, 26));
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Constructor> aConstructors = new Vector<>();
		final Vector<Constructor> bConstructors = new Vector<>();
		final Vector<Constructor> cConstructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, groupVector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name bName = group1Name.addName(bIdentifierToken);
		final Name cName = group1Name.addName(cIdentifierToken);
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						aConstructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType bClass =
				RegularClassType.create(
						bName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						bConstructors,
						
						bIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType cClass =
				RegularClassType.create(
						cName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						cConstructors,
						cIdentifierToken,
						
						new Vector<ClassType>());
		
		groupVector.add(aClass);
		groupVector.add(bClass);
		groupVector.add(cClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType bProduct = ProductType.create(this.bracketOpenToken);
		final ProductType cProduct = ProductType.create(this.bracketOpenToken);
		
		aConstructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		bConstructors.add(Constructor.create(bProduct, bClass, new Vector<ConstructorReference>(), bIdentifierToken));
		cConstructors.add(Constructor.create(cProduct, cClass, new Vector<ConstructorReference>(), cIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={();};];Group2:group=[B:class={();};];Group3.group=[C:class={();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testManyClassesWithOneConstructorDifferentGroups() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group1:group=[A:class={();};];Group2:group=[B:class={();};];Group3:group=[C:class={();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken group2IdentifierToken = IdentifierToken.create("Group2", Position.create("", 1, 31, 17));
		final IdentifierToken group3IdentifierToken = IdentifierToken.create("Group3", Position.create("", 1, 61, 34));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 15, 5));
		final IdentifierToken bIdentifierToken = IdentifierToken.create("B", Position.create("", 1, 45, 22));
		final IdentifierToken cIdentifierToken = IdentifierToken.create("C", Position.create("", 1, 75, 39));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<GroupElement> group2Vector = new Vector<>();
		final Vector<GroupElement> group3Vector = new Vector<>();
		final Vector<Constructor> aConstructors = new Vector<>();
		final Vector<Constructor> bConstructors = new Vector<>();
		final Vector<Constructor> cConstructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name group2Name = UnqualifiedName.create(group2IdentifierToken);
		final Group group2 = Group.create(group2Name, group2Vector, group2IdentifierToken);
		
		final Name group3Name = UnqualifiedName.create(group3IdentifierToken);
		final Group group3 = Group.create(group3Name, group3Vector, group3IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name bName = group2Name.addName(bIdentifierToken);
		final Name cName = group3Name.addName(cIdentifierToken);
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						aConstructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		final ClassType bClass =
				RegularClassType.create(
						bName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						bConstructors,
						bIdentifierToken,
						
						new Vector<ClassType>());
		final ClassType cClass =
				RegularClassType.create(
						cName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						cConstructors,
						cIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		group2Vector.add(bClass);
		group3Vector.add(cClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType bProduct = ProductType.create(this.bracketOpenToken);
		final ProductType cProduct = ProductType.create(this.bracketOpenToken);
		
		aConstructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		bConstructors.add(Constructor.create(bProduct, bClass, new Vector<ConstructorReference>(), bIdentifierToken));
		cConstructors.add(Constructor.create(cProduct, cClass, new Vector<ConstructorReference>(), cIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		expected.addGroup(group2);
		expected.addGroup(group3);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={();(x:Integer);(s:String);(x:String,s:String);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testClassWithManyConstructor() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={();(x:Integer);(s:String);(x:String,s:String);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken xParameterIdentifierToken = IdentifierToken.create("x", Position.create("", 1, 28, 15));
		final IdentifierToken sParameterIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType xProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		final ProductType xsProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final ByNameState stringName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		
		final TypeProxy stringTypeProxy = TypeProxy.create(this.stringIdentifier, stringName);
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		
		xProduct.addElement(ProductElementType.create("x", integerTypeProxy, xParameterIdentifierToken));
		xsProduct.addElement(ProductElementType.create("x", stringTypeProxy, sParameterIdentifierToken));
		final ProductElementType stringElement1 =
				ProductElementType.create("s", stringTypeProxy, sParameterIdentifierToken);
		sProduct.addElement(stringElement1);
		xsProduct.addElement(stringElement1);
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		constructors.add(Constructor.create(
				xProduct,
				aClass,
				new Vector<ConstructorReference>(),
				xParameterIdentifierToken));
		constructors.add(Constructor.create(
				sProduct,
				aClass,
				new Vector<ConstructorReference>(),
				sParameterIdentifierToken));
		constructors.add(Constructor.create(
				xsProduct,
				aClass,
				new Vector<ConstructorReference>(),
				xParameterIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[A:class={()=A(Integer);(i:Integer);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorReferenzesOtherConstructorInTheSameClass() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={()=A(Integer);(i:Integer);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iParameterIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 1));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType emptyProduct = ProductType.create(this.bracketOpenToken);
		final ProductType integerProduct = ProductType.create(this.bracketOpenToken);
		
		final List<ConstructorReference> references = new Vector<>();
		final List<Type> paramTypes = new Vector<>();
		
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final ByNameState aClassName = ByNameState.create(UnqualifiedName.create(aIdentifierToken));
		
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final TypeProxy aClassTypeProxy = TypeProxy.create(aIdentifierToken, aClassName);
		
		paramTypes.add(integerTypeProxy);
		
		references.add(ConstructorReference.create(ConstructorByTypeAndSignatureState.create(
				aClassTypeProxy,
				paramTypes)));
		
		constructors.add(Constructor.create(emptyProduct, aClass, references, this.bracketOpenToken));
		constructors.add(Constructor.create(
				integerProduct,
				aClass,
				new Vector<ConstructorReference>(),
				this.bracketOpenToken));
		
		integerProduct.addElement(ProductElementType.create("i", integerTypeProxy, iParameterIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[A:class={();op:[[(s:String)->{}]];};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstConstructorThanOperation() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={();op:[[(s:String)->{}]];};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sParameterIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState stringName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		
		final TypeProxy stringTypeProxy = TypeProxy.create(this.stringIdentifier, stringName);
		
		sProduct.addElement(ProductElementType.create("s", stringTypeProxy, sParameterIdentifierToken));
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Operation operation =
				Operation.create(
						"op",
						new Vector<OperationModifier>(),
						sProduct,
						aClass,
						SumType.create(this.curlyBracketOpenToken),
						op);
		operations.add(operation);
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={op:[[(s:String)->{}]];();};]; Parser macht wieder Reihenfolge: Konstruktor, Operation.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstOperationThanConstructor() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={op:[[(s:String)->{}]];();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sParameterIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState stringName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringTypeProxy = TypeProxy.create(this.stringIdentifier, stringName);
		
		sProduct.addElement(ProductElementType.create("s", stringTypeProxy, sParameterIdentifierToken));
		
		final Operation operation =
				Operation.create(
						"op",
						new Vector<OperationModifier>(),
						sProduct,
						aClass,
						SumType.create(this.curlyBracketOpenToken),
						op);
		operations.add(operation);
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={op1:[[(s:String)->{}]];();op2:[[(i:Integer)->{}]];};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorBetweenOperation() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={op1:[[(s:String)->{}]];();op2:[[(i:Integer)->{}]];};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sParameterIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken iParameterIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 1));
		
		final IdentifierToken op1 = IdentifierToken.create("op1", Position.create("", 1, 1, 0));
		final IdentifierToken op2 = IdentifierToken.create("op2", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState stringName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		
		final TypeProxy stringTypeProxy = TypeProxy.create(this.stringIdentifier, stringName);
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		
		sProduct.addElement(ProductElementType.create("s", stringTypeProxy, sParameterIdentifierToken));
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iParameterIdentifierToken));
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		
		operations.add(Operation.create("op1", new Vector<OperationModifier>(), sProduct, aClass, sum, op1));
		operations.add(Operation.create("op2", new Vector<OperationModifier>(), iProduct, aClass, sum, op2));
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={op1:[[(s:String)->{}]];();(i:Integer);(s:String);(x:String,s:String);op2:[[(i:Integer)->{}
	 * ]];};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testManyConstructorBetweenOperation() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={op1:[[(s:String)->{}]];();(i:Integer);"
						+ "(s:String);(x:String,s:String);op2:[[(i:Integer)->{}]];};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sParameterIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken iParameterIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 28, 15));
		
		final IdentifierToken op1 = IdentifierToken.create("op1", Position.create("", 1, 1, 0));
		final IdentifierToken op2 = IdentifierToken.create("op2", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType xProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		final ProductType xsProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final ByNameState stringName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		
		final TypeProxy stringTypeProxy = TypeProxy.create(this.stringIdentifier, stringName);
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		
		final ProductElementType stringElement1 =
				ProductElementType.create("s", stringTypeProxy, sParameterIdentifierToken);
		final ProductElementType integerElement =
				ProductElementType.create("i", integerTypeProxy, iParameterIdentifierToken);
		sProduct.addElement(stringElement1);
		xProduct.addElement(integerElement);
		xsProduct.addElement(ProductElementType.create("x", stringTypeProxy, sParameterIdentifierToken));
		xsProduct.addElement(stringElement1);
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		constructors.add(Constructor.create(
				xProduct,
				aClass,
				new Vector<ConstructorReference>(),
				iParameterIdentifierToken));
		constructors.add(Constructor.create(
				sProduct,
				aClass,
				new Vector<ConstructorReference>(),
				sParameterIdentifierToken));
		constructors.add(Constructor.create(
				xsProduct,
				aClass,
				new Vector<ConstructorReference>(),
				iParameterIdentifierToken));
		
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		iProduct.addElement(integerElement);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		
		operations.add(Operation.create("op1", new Vector<OperationModifier>(), sProduct, aClass, sum, op1));
		operations.add(Operation.create("op2", new Vector<OperationModifier>(), iProduct, aClass, sum, op2));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[A:class={s:String;();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstVariableThanConstructor() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={s:String;();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={();s:String;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstConstructorThanVariable() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={();s:String;};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={s:String;();i:Integer;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorBetweenVariable() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={s:String;();i:Integer;};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iParameterIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 1));
		final IdentifierToken nameIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final ByNameState attributeIntegerReferenceState =
				ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, attributeIntegerReferenceState);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		
		iProduct.addElement(ProductElementType.create("i", integerType, iParameterIdentifierToken));
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), nameIdentifierToken));
		attributes.add(Attribute.create("i", integerType, new Vector<AttributeModifier>(), iParameterIdentifierToken));
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={s:String;();(i:Integer);i:Integer;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testManyConstructorBetweenVariable() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={s:String;();(i:Integer);i:Integer;};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iParameterIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 1));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final ByNameState attributeIntegerReferenceState =
				ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, attributeIntegerReferenceState);
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		attributes.add(Attribute.create("i", integerType, new Vector<AttributeModifier>(), iParameterIdentifierToken));
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		
		iProduct.addElement(ProductElementType.create("i", integerType, iParameterIdentifierToken));
		
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		constructors.add(Constructor.create(iProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={op:[[(s:String)->{}]];s:String;();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstOperationThanVariableThanConstructor() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={op:[[(s:String)->{}]];s:String;();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		
		final TypeProxy stringType =
				TypeProxy.create(
						this.stringIdentifier,
						ByNameState.create(UnqualifiedName.create(this.stringIdentifier)));
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		sProduct.addElement(ProductElementType.create("s", stringType, sIdentifierToken));
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Operation operation =
				Operation.create(
						"op",
						new Vector<OperationModifier>(),
						sProduct,
						aClass,
						SumType.create(this.curlyBracketOpenToken),
						op);
		operations.add(operation);
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[A:class={op:[[(s:String)->{}]];();s:String;};];.
	 * 
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstOperationThanConstructorThanVariale() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={op:[[(s:String)->{}]];();s:String;};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		sProduct.addElement(ProductElementType.create("s", stringType, sIdentifierToken));
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Operation operation1 =
				Operation.create(
						"op",
						new Vector<OperationModifier>(),
						sProduct,
						aClass,
						SumType.create(this.curlyBracketOpenToken),
						op);
		operations.add(operation1);
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={();op:[[(s:String)->{}]];s:String;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testFirstConstructorThanOperationThanVariable() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class={();op:[[(s:String)->{}]];s:String;};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		sProduct.addElement(ProductElementType.create("s", stringType, sIdentifierToken));
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Operation operation =
				Operation.create(
						"op",
						new Vector<OperationModifier>(),
						sProduct,
						aClass,
						SumType.create(this.curlyBracketOpenToken),
						op);
		operations.add(operation);
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[SA:class={};A:class=SA+{();op:[[(s:String)->{}]];s:String;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithoutParameterAndSuperclassWithoutParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[SA:class={};A:class=SA+{();op:[[(s:String)->{}]];s:String;};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken saIdentifierToken = IdentifierToken.create("SA", Position.create("", 1, 1, 0));
		final IdentifierToken sIdentifierToken = IdentifierToken.create("s", Position.create("", 1, 1, 1));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name saName = group1Name.addName(saIdentifierToken);
		
		final ClassType saClass =
				RegularClassType.create(
						saName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						saIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(saClass);
		
		final TypeProxy supertype =
				TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(saIdentifierToken)));
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(supertype);
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						attributes,
						supertypes,
						operations,
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		final ProductType sProduct = ProductType.create(this.bracketOpenToken);
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		
		attributes.add(Attribute.create("s", stringType, new Vector<AttributeModifier>(), sIdentifierToken));
		sProduct.addElement(ProductElementType.create("s", stringType, sIdentifierToken));
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		final Operation operation =
				Operation.create(
						"op",
						new Vector<OperationModifier>(),
						sProduct,
						aClass,
						SumType.create(this.curlyBracketOpenToken),
						op);
		operations.add(operation);
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[SA:class={};A:class=SA+{(i:Integer);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithParameterAndSuperclassWithoutParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[SA:class={};A:class=SA+{(i:Integer);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken saIdentifierToken = IdentifierToken.create("SA", Position.create("", 1, 1, 0));
		final IdentifierToken iParameterIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 1));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name saName = group1Name.addName(saIdentifierToken);
		
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(saIdentifierToken))));
		
		final ClassType saClass =
				RegularClassType.create(
						saName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						saIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(saClass);
		group1Vector.add(aClass);
		
		// constructors
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iParameterIdentifierToken));
		constructors.add(Constructor.create(iProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		// equals
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[S1A:class={};S2A:class={};A:class=S1A+S2A+{();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithoutParameterAndManySuperclassesWithoutParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[S1A:class={};S2A:class={};A:class=S1A+S2A+{();};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken s1aIdentifierToken = IdentifierToken.create("S1A", Position.create("", 1, 1, 0));
		final IdentifierToken s2aIdentifierToken = IdentifierToken.create("S2A", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name s1aName = group1Name.addName(s1aIdentifierToken);
		final Name s2aName = group1Name.addName(s2aIdentifierToken);
		
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(s1aIdentifierToken))));
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(s2aIdentifierToken))));
		
		final ClassType s1aClass =
				RegularClassType.create(
						s1aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						s1aIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType s2aClass =
				RegularClassType.create(
						s2aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						s2aIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(s1aClass);
		group1Vector.add(s2aClass);
		group1Vector.add(aClass);
		
		final ProductType aProduct = ProductType.create(this.bracketOpenToken);
		constructors.add(Constructor.create(aProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		
		// equals
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[S1A:class={};S2A:class={};A:class=S1A+S2A+{(i:Integer);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithParameterAndManySuperclassesWithoutParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[S1A:class={};S2A:class={};A:class=S1A+S2A+{(i:Integer);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken s1aIdentifierToken = IdentifierToken.create("S1A", Position.create("", 1, 1, 0));
		final IdentifierToken s2aIdentifierToken = IdentifierToken.create("S2A", Position.create("", 1, 1, 0));
		final IdentifierToken iIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name s1aName = group1Name.addName(s1aIdentifierToken);
		final Name s2aName = group1Name.addName(s2aIdentifierToken);
		
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(s1aIdentifierToken))));
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(s2aIdentifierToken))));
		
		final ClassType s1aClass =
				RegularClassType.create(
						s1aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						s1aIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType s2aClass =
				RegularClassType.create(
						s2aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						s2aIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(s1aClass);
		group1Vector.add(s2aClass);
		group1Vector.add(aClass);
		
		// constructors
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iIdentifierToken));
		constructors.add(Constructor.create(iProduct, aClass, new Vector<ConstructorReference>(), aIdentifierToken));
		// equals
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[SA:class={(i:Integer);};A:class=SA+{();};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithoutParameterAndSuperclassWithParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[SA:class={(i:Integer);};A:class=SA+{};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken saIdentifierToken = IdentifierToken.create("SA", Position.create("", 1, 1, 0));
		final IdentifierToken iIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		
		final Name aName = group1Name.addName(aIdentifierToken);
		final Name saName = group1Name.addName(saIdentifierToken);
		
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(saIdentifierToken))));
		
		final ClassType saClass =
				RegularClassType.create(
						saName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						saIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						new Vector<Constructor>(),
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(saClass);
		group1Vector.add(aClass);
		
		// constructors
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iIdentifierToken));
		constructors.add(Constructor.create(iProduct, saClass, new Vector<ConstructorReference>(), saIdentifierToken));
		// equals
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		
		assertEquals(0, this.parser.getExceptions().size());
		
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[SA1:class={(i:Integer);};SA2:class={(i:Integer);};A:class=SA1+SA2+{(i:Integer)=SA1(Integer)+SA2(
	 * Integer);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithParameterDelegatesParameterToManySuperclassesWithParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[SA1:class={(i:Integer);};SA2:class={(i:Integer);};"
						+ "A:class=SA1+SA2+{(i:Integer)=SA1(Integer)+SA2(Integer);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 0));
		final IdentifierToken sa1IdentifierToken = IdentifierToken.create("SA1", Position.create("", 1, 1, 0));
		final IdentifierToken sa2IdentifierToken = IdentifierToken.create("SA2", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors1 = new Vector<>();
		final Vector<Constructor> constructors2 = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		final Name aName = group1Name.addName(aIdentifierToken);
		
		final Name sa1Name = group1Name.addName(sa1IdentifierToken);
		final Name sa2Name = group1Name.addName(sa2IdentifierToken);
		
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(sa1IdentifierToken))));
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(sa2IdentifierToken))));
		
		final ClassType sa1Class =
				RegularClassType.create(
						sa1Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors1,
						sa1IdentifierToken,
						
						new Vector<ClassType>());
		final ClassType sa2Class =
				RegularClassType.create(
						sa2Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors1,
						sa2IdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						constructors2,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(sa1Class);
		group1Vector.add(sa2Class);
		
		group1Vector.add(aClass);
		
		final List<ConstructorReference> references = new Vector<>();
		final List<Type> paramTypes = new Vector<>();
		
		final ByNameState sa1ClassName = ByNameState.create(UnqualifiedName.create(sa1IdentifierToken));
		final ByNameState sa2ClassName = ByNameState.create(UnqualifiedName.create(sa2IdentifierToken));
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final TypeProxy sa1ClassTypeProxy = TypeProxy.create(sa1IdentifierToken, sa1ClassName);
		final TypeProxy sa2ClassTypeProxy = TypeProxy.create(sa2IdentifierToken, sa2ClassName);
		
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		constructors1.add(Constructor.create(
				iProduct,
				sa2Class,
				new Vector<ConstructorReference>(),
				this.bracketOpenToken));
		constructors2.add(Constructor.create(iProduct, aClass, references, this.bracketOpenToken));
		
		references.add(ConstructorReference.create(ConstructorByTypeAndSignatureState.create(
				sa1ClassTypeProxy,
				paramTypes)));
		references.add(ConstructorReference.create(ConstructorByTypeAndSignatureState.create(
				sa2ClassTypeProxy,
				paramTypes)));
		paramTypes.add(integerTypeProxy);
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
		
	}
	
	/**
	 * Group1:group=[SA1:class={(i:Integer);};A:class=SA1+{(i:Integer)=SA1(Integer);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 * 
	 */
	@Test
	public void testConstructorWithParameterDelegatesParameterToOneSuperclassWithParameter() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group1:group=[SA1:class={(i:Integer);};A:class=SA1+{(i:Integer)=SA1(Integer);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 0));
		final IdentifierToken sa1IdentifierToken = IdentifierToken.create("SA1", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors1 = new Vector<>();
		final Vector<Constructor> constructors2 = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		final Name aName = group1Name.addName(aIdentifierToken);
		
		final Name sa1Name = group1Name.addName(sa1IdentifierToken);
		
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(sa1IdentifierToken))));
		
		final ClassType sa1Class =
				RegularClassType.create(
						sa1Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors1,
						sa1IdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						constructors2,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(sa1Class);
		group1Vector.add(aClass);
		
		final List<ConstructorReference> references = new Vector<>();
		final List<Type> paramTypes = new Vector<>();
		
		final ByNameState sa1ClassName = ByNameState.create(UnqualifiedName.create(sa1IdentifierToken));
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final TypeProxy sa1ClassTypeProxy = TypeProxy.create(sa1IdentifierToken, sa1ClassName);
		
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		constructors1.add(Constructor.create(
				iProduct,
				sa1Class,
				new Vector<ConstructorReference>(),
				this.bracketOpenToken));
		constructors2.add(Constructor.create(iProduct, aClass, references, this.bracketOpenToken));
		
		references.add(ConstructorReference.create(ConstructorByTypeAndSignatureState.create(
				sa1ClassTypeProxy,
				paramTypes)));
		paramTypes.add(integerTypeProxy);
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class=SA+{(i:Integer)=SA(Integer);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorWithoutParameterCallEmptySuperConstructorsWhichNotExist() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[A:class=SA+{(i:Integer)=SA(Integer);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 0));
		final IdentifierToken saIdentifierToken = IdentifierToken.create("SA", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		final Name aName = group1Name.addName(aIdentifierToken);
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(saIdentifierToken))));
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final List<ConstructorReference> references = new Vector<>();
		final List<Type> paramTypes = new Vector<>();
		
		final ByNameState saClassName = ByNameState.create(UnqualifiedName.create(saIdentifierToken));
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final TypeProxy saClassTypeProxy = TypeProxy.create(saIdentifierToken, saClassName);
		
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		constructors.add(Constructor.create(iProduct, aClass, references, this.bracketOpenToken));
		references.add(ConstructorReference.create(ConstructorByTypeAndSignatureState.create(
				saClassTypeProxy,
				paramTypes)));
		paramTypes.add(integerTypeProxy);
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group1:group=[A:class={(i:Integer)=A(String);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testConstructorCallsConstructorWhichNotExist() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[A:class={(i:Integer)=A(String);};];");
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final IdentifierToken group1IdentifierToken = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken aIdentifierToken = IdentifierToken.create("A", Position.create("", 1, 1, 0));
		final IdentifierToken iIdentifierToken = IdentifierToken.create("i", Position.create("", 1, 1, 0));
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<Constructor> constructors = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Group group1 = Group.create(group1Name, group1Vector, group1IdentifierToken);
		final Name aName = group1Name.addName(aIdentifierToken);
		
		final ClassType aClass =
				RegularClassType.create(
						aName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						constructors,
						aIdentifierToken,
						
						new Vector<ClassType>());
		group1Vector.add(aClass);
		
		final List<ConstructorReference> references = new Vector<>();
		final List<Type> paramTypes = new Vector<>();
		
		final ByNameState integerName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final ByNameState stringNameState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final ByNameState aNameState = ByNameState.create(UnqualifiedName.create(aIdentifierToken));
		
		final TypeProxy aTypeProxy = TypeProxy.create(aIdentifierToken, aNameState);
		final TypeProxy integerTypeProxy = TypeProxy.create(this.integerIdentifier, integerName);
		final TypeProxy stringTypeProxy = TypeProxy.create(this.stringIdentifier, stringNameState);
		
		final ProductType iProduct = ProductType.create(this.bracketOpenToken);
		constructors.add(Constructor.create(iProduct, aClass, references, this.bracketOpenToken));
		
		references.add(ConstructorReference.create(ConstructorByTypeAndSignatureState.create(aTypeProxy, paramTypes)));
		paramTypes.add(stringTypeProxy);
		
		iProduct.addElement(ProductElementType.create("i", integerTypeProxy, iIdentifierToken));
		
		final Model expected = Model.create(group1IdentifierToken);
		expected.addGroup(group1);
		
		this.parser = Parser.create(output);
		final Model actual = this.parser.parse();
		
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
}
