package de.fhdw.wtf.walker.tasks.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.AbstractOperationsException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.AbstractOperationsCheck;

/**
 * Tests the {@link AbstractOperationsCheck}.
 */
public class TestAbstractOperationsCheck extends TestCase {
	
	/**
	 * Group:group=[Klasse:class={name : String; operation : [[(parameter:Integer) -> String ]] abstract;}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={name : String; operation : [[(parameter:Integer) -> String ]] abstract;}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 43, 42), ex.getStartPosition());
			assertEquals(Position.create("", 1, 99, 98), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;}abstract;
	 * ];Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass2() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;}abstract; ];Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 124, 123), ex.getStartPosition());
			assertEquals(Position.create("", 1, 180, 179), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;}; Klasse2:class={operation :
	 * [[(parameter:Integer) -> String ]] abstract;}abstract;]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass3() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;}; Klasse2:class={operation : [[(parameter:Integer) -> String ]] abstract;}abstract;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 28, 27), ex.getStartPosition());
			assertEquals(Position.create("", 1, 84, 83), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] ;}; Klasse2:class={operation :
	 * [[(parameter:Integer) -> String ]] abstract;};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass4() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] ;}; Klasse2:class={operation : [[(parameter:Integer) -> String ]] abstract;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 94, 93), ex.getStartPosition());
			assertEquals(Position.create("", 1, 150, 149), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];operation2 :[[(parameter:Integer) ->
	 * String ]] abstract;}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass5() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];operation2 :[[(parameter:Integer) -> String ]] abstract;}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 75, 74), ex.getStartPosition());
			assertEquals(Position.create("", 1, 131, 130), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;operation2 :
	 * [[(parameter:Integer) -> String ]] abstract;}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass6() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;operation2 : [[(parameter:Integer) -> String ]] abstract;}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 28, 27), ex.getStartPosition());
			assertEquals(Position.create("", 1, 84, 83), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;} abstract;
	 * Klasse2:class={operation : [[(parameter:Integer) -> String ]] abstract;};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInNotAbstractClass7() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;} abstract; Klasse2:class={operation : [[(parameter:Integer) -> String ]] abstract;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof AbstractOperationsException) {
			final AbstractOperationsException ex = (AbstractOperationsException) exception;
			assertEquals(Position.create("", 1, 111, 110), ex.getStartPosition());
			assertEquals(Position.create("", 1, 167, 166), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;} abstract; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInAbstractClass() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] abstract;} abstract; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={name : String;operation : [[(parameter:Integer) -> String ]] ;operation2 :
	 * [[(parameter:Integer) -> String ]] abstract;} abstract; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInAbstractClass1() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={name : String;operation : [[(parameter:Integer) -> String ]] ;operation2 : [[(parameter:Integer) -> String ]] abstract;} abstract; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] ;operation2 : [[(parameter:Integer) ->
	 * String ]] ;} ; Klasse2:class={operation : [[(parameter:Integer) -> String ]] ;operation2 : [[(parameter:Integer)
	 * -> String ]] abstract;} abstract; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testAbstractOperationInAbstractClass2() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]] ;"
						+ "operation2 : [[(parameter:Integer) -> String ]] ;} ; "
						+ "Klasse2:class={operation : [[(parameter:Integer) -> String ]] ;"
						+ "operation2 : [[(parameter:Integer) -> String ]] abstract;} abstract; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		AbstractOperationsCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
}
