package de.fhdw.wtf.walker.tasks.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.BaseTypeInheritanceException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.BaseTypeInheritanceCheck;
import de.fhdw.wtf.walker.tasks.TypeReferencer;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

/**
 * Tests for {@link de.fhdw.wtf.walker.tasks.BaseTypeInheritanceCheck}.
 * 
 */
public class TestBaseTypeInheritanceCheck extends TestCase {
	
	/**
	 * personGroup:group=[Person:class=Integer+{};];.
	 * 
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 */
	@Test
	public void testInheritFromInteger() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input = new SimpleScannerInput("personGroup:group=[Person:class=Integer+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		baseTypeInheritanceCheck.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof BaseTypeInheritanceException) {
			final BaseTypeInheritanceException ex = (BaseTypeInheritanceException) exception;
			assertEquals(Position.create("", 1, 33, 32), ex.getStartPosition());
			assertEquals(Position.create("", 1, 40, 39), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * personGroup:group=[Person:class=String+{};];.
	 * 
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 */
	@Test
	public void testInheritFromString() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input = new SimpleScannerInput("personGroup:group=[Person:class=String+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		baseTypeInheritanceCheck.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof BaseTypeInheritanceException) {
			final BaseTypeInheritanceException ex = (BaseTypeInheritanceException) exception;
			assertEquals(Position.create("", 1, 33, 32), ex.getStartPosition());
			assertEquals(Position.create("", 1, 39, 38), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * personGroup:group=[Person:class=Anything+{};];.
	 * 
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 */
	@Test
	public void testInheritFromAnything() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input = new SimpleScannerInput("personGroup:group=[Person:class=Anything+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		baseTypeInheritanceCheck.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * personGroup:group=[Person:class=Person+{};];.
	 * 
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 */
	@Test
	public void testInheritFromItself() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input = new SimpleScannerInput("personGroup:group=[Person:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		baseTypeInheritanceCheck.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * personGroup:group=[Person:class=Object+{}; Object:class={};];.
	 * 
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 */
	@Test
	public void testInheritFromOtherClass() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=Object+{}; Object:class={};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask baseTypeInheritanceCheck = BaseTypeInheritanceCheck.create(model, taskmanager);
		baseTypeInheritanceCheck.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
}
