package de.fhdw.wtf.walker.tasks.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutionException;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.type.BaseType;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.CyclicInheritanceCheck;
import de.fhdw.wtf.walker.tasks.PrototypesTask;
import de.fhdw.wtf.walker.tasks.SubtypesFillTask;
import de.fhdw.wtf.walker.tasks.TypeReferencer;
import de.fhdw.wtf.walker.walker.HelperUtils;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

/**
 * Tests that every {@link Type} has a prototype so that no duplicates are used.
 */
public class TestPrototypesTask {
	
	/**
	 * Scanner, Parser, TypeReferencer und PrototypesTask laufen durch und liefern das Model.
	 * 
	 * @param input
	 * @return
	 * @throws CyclicDependencyException
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 */
	private Model getModel(final SimpleScannerInput input) throws CyclicDependencyException, InterruptedException,
			ExecutionException, NoValidTokenStreamException {
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		assertEquals(0, parser.getExceptions().size());
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(4, okResult.size());
		assertEquals(0, failResult.size());
		
		return model;
	}
	
	/**
	 * Group:group=[Class:class={Attribute:(name:String,count:Integer)*; Attribute2:(name:String,count:Integer)*;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTwoListsOfTheSameProduct() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute:(name:String,count:Integer)*; "
						+ "Attribute2:(name:String,count:Integer)*;};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> iterator = cls.getAttributes().iterator();
		final Attribute attribute_1 = iterator.next();
		final Attribute attribute_2 = iterator.next();
		
		assertTrue(((ListType) attribute_1.getAttrType()).getPrototype() != null
				&& ((ListType) attribute_1.getAttrType()).getPrototype() == ((ListType) attribute_2.getAttrType())
						.getPrototype());
		assertTrue(((ProductType) ((ListType) attribute_1.getAttrType()).getOf()).getPrototype() != null
				&& ((ProductType) ((ListType) attribute_1.getAttrType()).getOf()).getPrototype() == ((ProductType) ((ListType) attribute_2
						.getAttrType()).getOf()).getPrototype());
		
		final ProductType productPrototype =
				((ProductType) ((ListType) attribute_1.getAttrType()).getOf()).getPrototype();
		final ProductType productOrig = (ProductType) ((ListType) attribute_1.getAttrType()).getOf();
		final Iterator<ProductElementType> iteratorPrototyp = productPrototype.getElements().iterator();
		for (final ProductElementType current : productOrig.sortElements()) {
			assertTrue(current.equals(iteratorPrototyp.next()));
		}
		
		final ProductType productOrig2 = (ProductType) ((ListType) attribute_2.getAttrType()).getOf();
		final Iterator<ProductElementType> iteratorPrototyp2 = productPrototype.getElements().iterator();
		for (final ProductElementType current : productOrig2.sortElements()) {
			assertTrue(current.equals(iteratorPrototyp2.next()));
		}
		
		assertTrue(productOrig.getAbstractPrototype() != null);
		assertTrue(productOrig2.getAbstractPrototype() != null);
		assertTrue(productPrototype.getAbstractPrototype() != null);
		assertTrue(productOrig.getAbstractPrototype() == productOrig2.getAbstractPrototype());
		assertTrue(productPrototype.getAbstractPrototype() == productOrig.getAbstractPrototype());
		assertTrue(productPrototype.getAbstractPrototype() == productPrototype.getAbstractPrototype()
				.getAbstractPrototype());
		assertTrue(productPrototype.getAbstractPrototype() == productPrototype.getAbstractPrototype().getPrototype());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:String;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testAnything() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute:String;};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		assertTrue(model.getAnything().getPrototype() != null);
	}
	
	/**
	 * Group:group=[Class:class={Attribute:{Anything,String};};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testAnything2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute:{Anything,String};};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		assertTrue(model.getAnything().getPrototype() != null);
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute_1 = attributes.next();
		
		assertTrue(attribute_1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute_1.getAttrType();
		assertTrue(sum.getPrototype() != null);
		assertTrue(model.getAnything().getPrototype() == sum.getPrototype());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:Anything;};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testAnything3() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute:Anything;};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		assertTrue(model.getAnything().getPrototype() != null);
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:[Integer->String]; Attribute2:(name:String, count:Integer);
	 * Attribute3:[Integer->String];};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTwoEqualMapsAndOneOtherType() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:[Integer->String]; "
						+ "Attribute2:(name:String, count:Integer); Attribute3:[Integer->String];};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute_1 = attributes.next();
		final Attribute attribute_2 = attributes.next();
		final Attribute attribute_3 = attributes.next();
		
		assertTrue(attribute_1.getAttrType() instanceof MapType);
		final MapType map1 = (MapType) attribute_1.getAttrType();
		assertTrue(attribute_2.getAttrType() instanceof ProductType);
		final ProductType product = (ProductType) attribute_2.getAttrType();
		assertTrue(attribute_3.getAttrType() instanceof MapType);
		final MapType map3 = (MapType) attribute_3.getAttrType();
		
		assertTrue(map1.getPrototype() == map3.getPrototype());
		assertNotNull(product.getPrototype());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{String, Integer}; Attribute2:{Integer, String};};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTwoEqualSums() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Class:class={Attribute1:{String, Integer}; Attribute2:{Integer, String};};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute_1 = attributes.next();
		final Attribute attribute_2 = attributes.next();
		
		assertTrue(attribute_1.getAttrType() instanceof SumType);
		final SumType sum1 = (SumType) attribute_1.getAttrType();
		assertTrue(attribute_2.getAttrType() instanceof SumType);
		final SumType sum2 = (SumType) attribute_2.getAttrType();
		
		assertTrue(sum1.getPrototype() == sum2.getPrototype());
		
		final List<Type> sorted = sum1.sortElements();
		assertEquals(sorted, ((SumType) sum2.getPrototype()).getElements());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{Integer};};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningInteger() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute1:{Integer};};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() instanceof BaseType);
		final BaseType integer = (BaseType) sum.getPrototype();
		
		assertTrue(integer == model.getInteger());
		// String,Integer,Class
		assertEquals(3, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{String};};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningString() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute1:{String};};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() instanceof BaseType);
		final BaseType string = (BaseType) sum.getPrototype();
		
		assertTrue(string == model.getString());
		// String,Integer,Class
		assertEquals(3, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{Class};};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningOneClass() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute1:{Class};};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() == cls);
		// String,Integer,Class
		assertEquals(3, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{{Class}};};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningOneSumWithOneClass() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute1:{{Class}};};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() == cls);
		// String,Integer,Class
		assertEquals(3, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{String*}; Attribute2:String*;};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningOneList() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:{String*}; Attribute2:String*;};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		final Attribute attribute2 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() instanceof ListType);
		final ListType list = (ListType) sum.getPrototype();
		
		assertTrue(list.getPrototype() == list);
		assertTrue(list == ((ListType) attribute2.getAttrType()).getPrototype());
		// String,Integer,Class and the List
		assertEquals(4, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{(name:String,alter:Integer)}; Attribute2:(name:String,alter:Integer);};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningOneProduct() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:{(name:String,alter:Integer)}; "
						+ "Attribute2:(name:String,alter:Integer);};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		final Attribute attribute2 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() instanceof ProductType);
		final ProductType product = (ProductType) sum.getPrototype();
		
		assertTrue(product.getPrototype() == product);
		assertTrue(product == ((ProductType) attribute2.getAttrType()).getPrototype());
		
		// String,Integer,Class and the Product + abstr. Product
		assertEquals(5, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{[String->Integer]}; Attribute2:[String->Integer];};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningOneMap() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:{[String->Integer]};"
						+ " Attribute2:[String->Integer];};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		final Attribute attribute2 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() instanceof MapType);
		final MapType product = (MapType) sum.getPrototype();
		
		assertTrue(product.getPrototype() == product);
		assertTrue(product == ((MapType) attribute2.getAttrType()).getPrototype());
		// String,Integer,Class and the Map
		assertEquals(4, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{Class};};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testSumWithOneElementAtBeginningOneClassAfterStandardize() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:{Class,Class2};};"
						+ " Class2:class=Class+{};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof SumType);
		final SumType sum = (SumType) attribute1.getAttrType();
		
		assertTrue(sum.getPrototype() == cls);
		// String,Integer,Class
		assertEquals(3, ((SumType) model.getAnything().getPrototype()).getElements().size());
	}
	
	/**
	 * Group:group=[Class1:class={Attribute7:String*;}; Class2:class={Attribute1:{Class1, Integer}; Attribute2:{Class1,
	 * String}; Attribute3:String* Attribute4:String* Attribute5:{Integer, Class1} Attribute6:{String, Class1}}; };]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testMoreEqualTypes() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Class1:class={Attribute7:String*;}; Class2:class={Attribute1:{Class1, Integer}; "
								+ "Attribute2:{Class1, String}; Attribute3:String*; Attribute4:String*; "
								+ "Attribute5:{Integer, Class1}; Attribute6:{String, Class1}; };];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final Iterator<GroupElement> groupElementIterator =
				model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType class1 = (ClassType) groupElementIterator.next();
		
		assertTrue(class1.getPrototype() == class1);
		
		final ClassType class2 = (ClassType) groupElementIterator.next();
		
		assertTrue(class2.getPrototype() == class2);
		
		final Iterator<Attribute> attributesClass1 = class1.getAttributes().iterator();
		final ListType attribute7 = (ListType) attributesClass1.next().getAttrType();
		
		final Iterator<Attribute> attributesClass2 = class2.getAttributes().iterator();
		final SumType attribute1 = (SumType) attributesClass2.next().getAttrType();
		final SumType attribute2 = (SumType) attributesClass2.next().getAttrType();
		final ListType attribute3 = (ListType) attributesClass2.next().getAttrType();
		final ListType attribute4 = (ListType) attributesClass2.next().getAttrType();
		final SumType attribute5 = (SumType) attributesClass2.next().getAttrType();
		final SumType attribute6 = (SumType) attributesClass2.next().getAttrType();
		
		assertTrue(attribute1.getPrototype() == attribute5.getPrototype());
		assertTrue(attribute2.getPrototype() == attribute6.getPrototype());
		assertTrue(attribute3.getPrototype() == attribute4.getPrototype());
		assertTrue(attribute3.getPrototype() == attribute7.getPrototype());
		
		final List<Type> sorted1 = attribute1.sortElements();
		assertEquals(sorted1, ((SumType) attribute1.getPrototype()).getElements());
		final List<Type> sorted2 = attribute2.sortElements();
		assertEquals(sorted2, ((SumType) attribute2.getPrototype()).getElements());
	}
	
	/**
	 * Group:group=[Class:class={operation1:[[(test1:String, test2:String)->Integer*]]; operation2:[[(test1:String,
	 * test2:String)->Integer*]];};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTwoOperationsWithSameParamsAndReturnValue() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						" Group:group=[Class:class={operation1:[[(test1:String, test2:String)->Integer*]]; operation2:[[(test1:String, test2:String)->Integer*]];};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final Iterator<GroupElement> groupElementIterator =
				model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType cls = (ClassType) groupElementIterator.next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Operation> operations = cls.getOperations().iterator();
		final Operation operation1 = operations.next();
		final Operation operation2 = operations.next();
		
		assertTrue(operation1.getParameters().getPrototype() == operation2.getParameters().getPrototype());
		assertTrue(operation1.getParameters().getPrototype() == operation2.getParameters().getPrototype());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:(sum1:{String, Integer}, sum2:{Integer, String}, map1:[String*->String*],
	 * map2:[String*->String*]); Attribute2:(sum1:{String, Integer}, sum2:{Integer, String}, map1:[String*->String*],
	 * map2:[String*->String*]);};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTwoComplexAttributesWithSameType() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:(sum1:{String, Integer}, "
						+ "sum2:{Integer, String}, map1:[String*->String*], map2:[String*->String*]); "
						+ "Attribute2:(sum1:{String, Integer}, sum2:{Integer, String}, map1:[String*->String*], "
						+ "map2:[String*->String*]);};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final Iterator<GroupElement> groupElementIterator =
				model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType cls = (ClassType) groupElementIterator.next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final ProductType product1 = (ProductType) attributes.next().getAttrType();
		final ProductType product2 = (ProductType) attributes.next().getAttrType();
		
		assertTrue(product1.getPrototype() == product2.getPrototype());
		
		final Iterator<ProductElementType> productElements1 = product1.getElements().iterator();
		final SumType sum11 = (SumType) productElements1.next().getType();
		final SumType sum12 = (SumType) productElements1.next().getType();
		final MapType map11 = (MapType) productElements1.next().getType();
		final MapType map12 = (MapType) productElements1.next().getType();
		
		assertTrue(sum11.getPrototype() == sum12.getPrototype());
		assertTrue(map11.getPrototype() == map12.getPrototype());
		
		final Iterator<ProductElementType> productElements2 = product2.getElements().iterator();
		final SumType sum21 = (SumType) productElements2.next().getType();
		final SumType sum22 = (SumType) productElements2.next().getType();
		final MapType map21 = (MapType) productElements2.next().getType();
		final MapType map22 = (MapType) productElements2.next().getType();
		
		assertTrue(sum21.getPrototype() == sum22.getPrototype());
		assertTrue(map12.getPrototype() == map22.getPrototype());
		assertTrue(sum11.getPrototype() == sum21.getPrototype());
		assertTrue(sum12.getPrototype() == sum22.getPrototype());
		assertTrue(map11.getPrototype() == map21.getPrototype());
		assertTrue(map12.getPrototype() == map22.getPrototype());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:Class;};]; .
	 * 
	 * @throws Exception
	 *             possible Exception
	 */
	@Test
	public void testTypeProxy() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute1:Class;};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> attributes = cls.getAttributes().iterator();
		final Attribute attribute1 = attributes.next();
		
		assertTrue(attribute1.getAttrType() instanceof TypeProxy);
		final TypeProxy typeProxy = (TypeProxy) attribute1.getAttrType();
		
		assertTrue(typeProxy.getPrototype() == typeProxy);
		
		assertTrue(HelperUtils.getTargetType(typeProxy) == cls);
	}
	
	/**
	 * Group:group=[Class:class={Attribute:(name:String,count:Integer); Attribute2:(p1:String,p2:Integer);};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testProduct() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute:(name:String,count:Integer); "
						+ "Attribute2:(p1:String,p2:Integer);};];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> iterator = cls.getAttributes().iterator();
		final Attribute attribute1 = iterator.next();
		final Attribute attribute2 = iterator.next();
		
		assertTrue(attribute1.getAttrType().getPrototype() != null);
		assertTrue(attribute2.getAttrType().getPrototype() != null);
		
		final ProductType product1 = (ProductType) attribute1.getAttrType();
		final ProductType product2 = (ProductType) attribute2.getAttrType();
		
		assertTrue(product1.getAbstractPrototype() != null);
		assertTrue(product2.getAbstractPrototype() != null);
		
		assertTrue(product1.getPrototype() != null);
		assertTrue(product2.getPrototype() != null);
		
		assertTrue(product1.getAbstractPrototype() == product1.getPrototype().getAbstractPrototype());
		assertTrue(product2.getAbstractPrototype() == product2.getPrototype().getAbstractPrototype());
		
		assertTrue(product1.getAbstractPrototype() == product2.getAbstractPrototype());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:(p1:String,p2:Integer);};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testProduct2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute:(p1:String,p2:Integer); };];");
		final Model model = this.getModel(input);
		
		assertTrue(model.getString().getPrototype() == model.getString());
		assertTrue(model.getInteger().getPrototype() == model.getInteger());
		
		final ClassType cls = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertTrue(cls.getPrototype() == cls);
		
		final Iterator<Attribute> iterator = cls.getAttributes().iterator();
		final Attribute attribute1 = iterator.next();
		
		assertTrue(attribute1.getAttrType().getPrototype() != null);
		
		final ProductType product1 = (ProductType) attribute1.getAttrType();
		
		assertTrue(product1.getAbstractPrototype() != null);
		assertTrue(product1.getPrototype() != null);
		
		assertTrue(product1.getAbstractPrototype() == product1.getPrototype().getAbstractPrototype());
	}
}
