package de.fhdw.wtf.walker.tasks.test;

import static org.junit.Assert.assertEquals;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.ExecutionException;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.QualifiedName;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.BaseType;
import de.fhdw.wtf.common.ast.type.ByReferenceState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.DummyToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.core.integration.testutil.AssertOperations;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.util.TypeReferencerTestReturnValue;

/**
 * Test the functions of the referencer: - Finding the right type for a given name in {@link Model}. - Eliminate
 * duplicate types. - Create anything.
 */
public class TestTypeReferencer {
	
	/**
	 * Contains different functions to check the results of referencing.
	 */
	private AssertOperations assertOperations;
	
	/**
	 * Sets up the {@link AssertOperations} for each testcase.
	 */
	@Before
	public void setUp() {
		this.assertOperations = new AssertOperations();
	}
	
	/**
	 * Test the referencing of: Group1:group=[Class1:class={name:String prior;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void simpleGroupWithClassAndOneAttribute() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[Class1:class={name:String prior;};];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(1, v.getOkResult().size());
		assertEquals(0, v.getFailResult().size());
		this.assertOperations.assertAllReferencesByType(result);
	}
	
	/**
	 * Test the referencing of: Group1:group=[Class1:class={name:String prior;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void simpleGroupInGroupAnd3ClassesWithDifferentAttributes() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[Class1:class={name:String prior;};"
						+ "Class2:class={type:Class1*;name:String prior;alter:Integer prior;};"
						+ "Group2:group=[Class1:class= Class2 + {beschreibungen:String * prior;};];];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(result);
	}
	
	/**
	 * Test the referencing of: Group1:group=[Class1:class={name:String prior;}; Class2:class={type:Class1*;name:String
	 * prior;alter:Integer prior;};]; Group2:group=[Class1:class= Class2 + {beschreibungen:String * prior;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void simple2GroupsAnd3ClassesWithDifferentAttributesAndAWrongQualification() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[Class1:class={name:String prior;};"
						+ "Class2:class={type:Class1*;name:String prior;alter:Integer prior;};];"
						+ "Group2:group=[Class1:class= Class2 + {beschreibungen:String * prior;};];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(0, v.getOkResult().size());
	}
	
	/**
	 * Test the referencing of: Group1:group=[Class1:class={name:String prior;}; Class2:class={type:Class1*;name:String
	 * prior;alter:Integer prior;}; Group2:group=[Class1:class= Class2 + Class2 + {beschreibungen:String * prior;};];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void simple2GroupsAnd3ClassesWithDifferentAttributesAnd2Supertypes() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[Class1:class={name:String prior;};"
						+ "Class2:class={type:Class1*;name:String prior;alter:Integer prior;};"
						+ "Group2:group=[Class1:class= Class2 + Class2 + {beschreibungen:String * prior;};];];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(result);
	}
	
	/**
	 * Test the referencing of:Group1:group=[Class1:class={name:String prior;};Class2:class={type:Class1*;name:String
	 * prior;alter:Integer prior;}abstract;Group2:group=[ Class1:class= Class2 + Class2 + {beschreibungen:String *
	 * prior;};Class1:class = {name:String prior;}; Group3:group=[Class1:class= Class1 + {alter:Integer;};];* ];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void test3GroupsAndManyClassesWithDifferentAttributes() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[Class1:class={"
						+ "name:String prior;};Class2:class={type:Class1*;name:String prior;"
						+ "alter:Integer prior;}abstract;Group2:group=[Class1:class= Class2 + Class2 + {"
						+ "beschreibungen:String * prior;}; Class1:class = {name:String prior;};"
						+ "Group3:group=[Class1:class= Class1 + {alter:Integer;};];];];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(result);
	}
	
	/**
	 * Test the referencing of:Group1:group=[Group1:group=[Class1:class={};];Class1:class= Group1>Group1>Class1 + {};];
	 * Group2:group=[Class2:class = Group1>Class1 + Group1>Group1>Class1 + {class1:Group1>Class1* prior;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSupertypesWithDifferentGroups() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[Group1:group=[Class1:class={"
						+ "};];Class1:class= Group1>Group1>Class1 + {};];Group2:group=["
						+ "Class2:class = Group1>Class1 + Group1>Group1>Class1 + {class1:Group1>Class1* prior;};];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(1, v.getFailResult().size());
		// TODO
		// assertEquals(3, v.getTypeReferencer().getExceptions().size());
		assertEquals(0, v.getOkResult().size());
	}
	
	/**
	 * Test the referencing of:Group1:group = [Group2:group = [Class1:class={};];];Group2:group=
	 * [Class1:class={};Group1:group=[Class1:class=
	 * Group2>Class1+{classa:Group1>Group2>Class1;};Group1:group=[Class1:class = Group2>Group1>Class1 +
	 * {class21:Group2>Group1>Class1;};];];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void test2GroupsInModelAndDifferentOtherGroupsCheckingRelativePath() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group = [Group2:group = ["
						+ "Class1:class={};];];Group2:group= [Class1:class={};Group1:group=["
						+ "Class1:class= Group2>Class1 +{classa:Group1>Group2>Class1;};Group1:group=["
						+ "Class1:class = Group2>Group1>Class1 + {class21:Group2>Group1>Class1;};];];];");
		final IdentifierToken group1IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Model model = Model.create(group1IdentifierToken, null);
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Collection<GroupElement> group1Elements = new Vector<>();
		final Group group1 = Group.create(group1Name, group1Elements, group1IdentifierToken, null);
		model.addGroup(group1);
		final IdentifierToken group12IdentifierToken =
				IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group12Name = group1Name.addName(group12IdentifierToken);
		final Collection<GroupElement> group12Elements = new Vector<>();
		final Group group12 = Group.create(group12Name, group12Elements, group12IdentifierToken, null);
		group1Elements.add(group12);
		final IdentifierToken class121IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class121Name = group12Name.addName(class121IdentifierToken);
		final Collection<ClassModifier> class121Modifiers = new Vector<>();
		final Collection<Attribute> class121Attributes = new Vector<>();
		final List<Type> class121SuperTypes = new Vector<>();
		final Collection<Operation> class121Operations = new Vector<>();
		final ClassType class121 =
				RegularClassType.create(
						class121Name,
						class121Modifiers,
						class121Attributes,
						class121SuperTypes,
						class121Operations,
						new Vector<Constructor>(),
						class121IdentifierToken,
						
						new Vector<ClassType>(),
						null);
		group12Elements.add(class121);
		final IdentifierToken group2IdentifierToken =
				IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group2Name = UnqualifiedName.create(group2IdentifierToken);
		final Collection<GroupElement> group2Elements = new Vector<>();
		final Group group2 = Group.create(group2Name, group2Elements, group2IdentifierToken, null);
		model.addGroup(group2);
		final IdentifierToken class21IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class21Name = group2Name.addName(class21IdentifierToken);
		final Collection<ClassModifier> class21Modifiers = new Vector<>();
		final Collection<Attribute> class21Attributes = new Vector<>();
		final List<Type> class21SuperTypes = new Vector<>();
		final Collection<Operation> class21Operations = new Vector<>();
		final ClassType class21 =
				RegularClassType.create(
						class21Name,
						class21Modifiers,
						class21Attributes,
						class21SuperTypes,
						class21Operations,
						new Vector<Constructor>(),
						class21IdentifierToken,
						
						new Vector<ClassType>(),
						null);
		group2Elements.add(class21);
		final IdentifierToken group21IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group21Name = group2Name.addName(group21IdentifierToken);
		final Collection<GroupElement> group21Elements = new Vector<>();
		final Group group21 = Group.create(group21Name, group21Elements, group21IdentifierToken, null);
		group2Elements.add(group21);
		final IdentifierToken class211IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class211Name = group21Name.addName(class211IdentifierToken);
		final Collection<ClassModifier> class211Modifiers = new Vector<>();
		final Collection<Attribute> class211Attributes = new Vector<>();
		final List<Type> class211SuperTypes = new Vector<>();
		final Collection<Operation> class211Operations = new Vector<>();
		final ClassType class211 =
				RegularClassType.create(
						class211Name,
						class211Modifiers,
						class211Attributes,
						class211SuperTypes,
						class211Operations,
						new Vector<Constructor>(),
						class211IdentifierToken,
						
						new Vector<ClassType>(),
						null);
		group21Elements.add(class211);
		final ByReferenceState class211SuperTypeState = ByReferenceState.create(class21, class21.getTypeName());
		final TypeProxy class211Supertype = TypeProxy.create(class211IdentifierToken, class211SuperTypeState, null);
		class211SuperTypes.add(class211Supertype);
		final ByReferenceState class211AttributeTypeReferenceState =
				ByReferenceState.create(class121, class121.getTypeName());
		final TypeProxy class211AttributeTypeReference =
				TypeProxy.create(class121IdentifierToken, class211AttributeTypeReferenceState, null);
		
		final IdentifierToken class211AttributeTypeIdentifier =
				IdentifierToken.create("", Position.create("", 0 + 1, 0 + 1, 0));
		final Attribute class211Attribute =
				Attribute.create(
						"classa",
						class211AttributeTypeReference,
						new Vector<AttributeModifier>(),
						class211IdentifierToken,
						null);
		class211Attributes.add(class211Attribute);
		final IdentifierToken group211IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group211Name = group21Name.addName(group211IdentifierToken);
		final Collection<GroupElement> group211Elements = new Vector<>();
		final Group group211 = Group.create(group211Name, group211Elements, group211IdentifierToken, null);
		group21Elements.add(group211);
		final IdentifierToken class2111IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class2111Name = group211Name.addName(class2111IdentifierToken);
		final Collection<ClassModifier> class2111Modifiers = new Vector<>();
		final Collection<Attribute> class2111Attributes = new Vector<>();
		final List<Type> class2111SuperTypes = new Vector<>();
		final Collection<Operation> class2111Operations = new Vector<>();
		final ClassType class2111 =
				RegularClassType.create(
						class2111Name,
						class2111Modifiers,
						class2111Attributes,
						class2111SuperTypes,
						class2111Operations,
						new Vector<Constructor>(),
						class211IdentifierToken,
						
						new Vector<ClassType>(),
						null);
		group211Elements.add(class2111);
		final ByReferenceState class2111SuperTypeState = ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class2111Supertype = TypeProxy.create(class2111IdentifierToken, class2111SuperTypeState, null);
		class2111SuperTypes.add(class2111Supertype);
		final ByReferenceState class2111AttributeTypeReferenceState =
				ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class2111AttributeTypeReference =
				TypeProxy.create(class211AttributeTypeIdentifier, class2111AttributeTypeReferenceState, null);
		final Attribute class2111Attribute =
				Attribute.create(
						"class21",
						class2111AttributeTypeReference,
						new Vector<AttributeModifier>(),
						class2111IdentifierToken,
						null);
		class2111Attributes.add(class2111Attribute);
		
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		
		final Model result = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(1, v.getTypeReferencer().getExceptions().size());
		assertEquals(0, v.getOkResult().size());
	}
	
	/**
	 * Test the referencing of:Group1:group = [Group2:group = [Class1:class={};];]; Group2:group= [Class1:class=
	 * Group1>Class1 + {};Group1:group=[Class1:class= {classa:Group1>Group2>Class1;};Group1:group=[Class1:class =
	 * Group2>Group1>Class1 + {class21:Group2>Group1>Class1;};];];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void test2GroupsWithClassesRelativePath() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group = [Group2:group = ["
						+ "Class1:class={};];]; Group2:group= [Class1:class= Group1>Class1 + {};"
						+ "Group1:group=[Class1:class= {classa:Group1>Group2>Class1;};Group1:group=["
						+ "Class1:class = Group2>Group1>Class1 + {class21:Group2>Group1>Class1;};];];];");
		final IdentifierToken group1IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Model model = Model.create(group1IdentifierToken, null);
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Collection<GroupElement> group1Elements = new Vector<>();
		final Group group1 = Group.create(group1Name, group1Elements, group1IdentifierToken, null);
		model.addGroup(group1);
		final IdentifierToken group12IdentifierToken =
				IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group12Name = group1Name.addName(group12IdentifierToken);
		final Collection<GroupElement> group12Elements = new Vector<>();
		final Group group12 = Group.create(group12Name, group12Elements, group12IdentifierToken, null);
		group1Elements.add(group12);
		final IdentifierToken class121IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class121Name = group12Name.addName(class121IdentifierToken);
		final Collection<ClassModifier> class121Modifiers = new Vector<>();
		final Collection<Attribute> class121Attributes = new Vector<>();
		final List<Type> class121SuperTypes = new Vector<>();
		final Collection<Operation> class121Operations = new Vector<>();
		final ClassType class121 =
				RegularClassType.create(
						class121Name,
						class121Modifiers,
						class121Attributes,
						class121SuperTypes,
						class121Operations,
						new Vector<Constructor>(),
						class121IdentifierToken,
						
						new Vector<ClassType>(),
						null);
		group12Elements.add(class121);
		final IdentifierToken group2IdentifierToken =
				IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group2Name = UnqualifiedName.create(group2IdentifierToken);
		final Collection<GroupElement> group2Elements = new Vector<>();
		final Group group2 = Group.create(group2Name, group2Elements, group2IdentifierToken, null);
		model.addGroup(group2);
		final IdentifierToken class21IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class21Name = group2Name.addName(class21IdentifierToken);
		final Collection<ClassModifier> class21Modifiers = new Vector<>();
		final Collection<Attribute> class21Attributes = new Vector<>();
		final List<Type> class21SuperTypes = new Vector<>();
		final Collection<Operation> class21Operations = new Vector<>();
		final ClassType class21 =
				RegularClassType.create(
						class21Name,
						class21Modifiers,
						class21Attributes,
						class21SuperTypes,
						class21Operations,
						new Vector<Constructor>(),
						class21IdentifierToken,
						
						new Vector<ClassType>(),
						null);
		group2Elements.add(class21);
		final IdentifierToken group21IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group21Name = group2Name.addName(group21IdentifierToken);
		final Collection<GroupElement> group21Elements = new Vector<>();
		final Group group21 = Group.create(group21Name, group21Elements, group21IdentifierToken);
		group2Elements.add(group21);
		final IdentifierToken class211IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class211Name = group21Name.addName(class211IdentifierToken);
		final Collection<ClassModifier> class211Modifiers = new Vector<>();
		final Collection<Attribute> class211Attributes = new Vector<>();
		final List<Type> class211SuperTypes = new Vector<>();
		final Collection<Operation> class211Operations = new Vector<>();
		final ClassType class211 =
				RegularClassType.create(
						class211Name,
						class211Modifiers,
						class211Attributes,
						class211SuperTypes,
						class211Operations,
						new Vector<Constructor>(),
						class211IdentifierToken,
						
						new Vector<ClassType>());
		group21Elements.add(class211);
		final ByReferenceState class21SuperTypeReferenceState =
				ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class21SuperTypeReference =
				TypeProxy.create(class211IdentifierToken, class21SuperTypeReferenceState);
		class21SuperTypes.add(class21SuperTypeReference);
		final ByReferenceState class211AttributeTypeReferenceState =
				ByReferenceState.create(class121, class121.getTypeName());
		final TypeProxy class211AttributeTypeReference =
				TypeProxy.create(class121IdentifierToken, class211AttributeTypeReferenceState);
		final IdentifierToken class211AttributeTypeIdentifier =
				IdentifierToken.create("", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken class211AttributeIdentifier =
				IdentifierToken.create("classa", Position.create("", 0 + 1, 0 + 1, 0));
		final Attribute class211Attribute =
				Attribute.create(
						"classa",
						class211AttributeTypeReference,
						new Vector<AttributeModifier>(),
						class211AttributeIdentifier);
		class211Attributes.add(class211Attribute);
		final IdentifierToken group211IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group211Name = group21Name.addName(group211IdentifierToken);
		final Collection<GroupElement> group211Elements = new Vector<>();
		final Group group211 = Group.create(group211Name, group211Elements, group211IdentifierToken);
		group21Elements.add(group211);
		final IdentifierToken class2111IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class2111Name = group211Name.addName(class2111IdentifierToken);
		final Collection<ClassModifier> class2111Modifiers = new Vector<>();
		final Collection<Attribute> class2111Attributes = new Vector<>();
		final List<Type> class2111SuperTypes = new Vector<>();
		final Collection<Operation> class2111Operations = new Vector<>();
		final ClassType class2111 =
				RegularClassType.create(
						class2111Name,
						class2111Modifiers,
						class2111Attributes,
						class2111SuperTypes,
						class2111Operations,
						new Vector<Constructor>(),
						class2111IdentifierToken,
						
						new Vector<ClassType>());
		group211Elements.add(class2111);
		final ByReferenceState class2111SuperTypeState = ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class2111Supertype = TypeProxy.create(class2111IdentifierToken, class2111SuperTypeState);
		class2111SuperTypes.add(class2111Supertype);
		final ByReferenceState class2111AttributeTypeReferenceState =
				ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class2111AttributeTypeReference =
				TypeProxy.create(class211AttributeTypeIdentifier, class2111AttributeTypeReferenceState);
		final IdentifierToken class2111AttributeIdentifier =
				IdentifierToken.create("", Position.create("", 0 + 1, 0 + 1, 0));
		final Attribute class2111Attribute =
				Attribute.create(
						"class21",
						class2111AttributeTypeReference,
						new Vector<AttributeModifier>(),
						class2111AttributeIdentifier);
		class2111Attributes.add(class2111Attribute);
		
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(1, v.getTypeReferencer().getExceptions().size());
		assertEquals(0, v.getOkResult().size());
	}
	
	/**
	 * Test the referencing of: Group1:group = [Group2:group = [Class1:class={};];]; Group2:group= [Class1:class=
	 * Group1>Class1 + {}; Group1:group=[Class1:class= Group2>Class1 +{classa:Group1>Group2>Class1;};
	 * Group1:group=[Class1:class = Group2>Group1>Class1 + {class21:Group2>Group1>Class1;};];];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void test2GroupsWithClasseHierarchieZyklus() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group = [Group2:group = ["
						+ "Class1:class={};];];Group2:group= [Class1:class= Group1>Class1 + {};"
						+ "Group1:group=[Class1:class= Group2>Class1 +{classa:Group1>Group2>Class1;};"
						+ "Group1:group=[Class1:class = Group2>Group1>Class1 + {class21:Group2>Group1>Class1;};];];];");
		final IdentifierToken group1IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Model model = Model.create(group1IdentifierToken);
		
		final Name group1Name = UnqualifiedName.create(group1IdentifierToken);
		final Collection<GroupElement> group1Elements = new Vector<>();
		final Group group1 = Group.create(group1Name, group1Elements, group1IdentifierToken);
		model.addGroup(group1);
		final IdentifierToken group12IdentifierToken =
				IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group12Name = group1Name.addName(group12IdentifierToken);
		final Collection<GroupElement> group12Elements = new Vector<>();
		final Group group12 = Group.create(group12Name, group12Elements, group12IdentifierToken);
		group1Elements.add(group12);
		final IdentifierToken class121IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class121Name = group12Name.addName(class121IdentifierToken);
		final Collection<ClassModifier> class121Modifiers = new Vector<>();
		final Collection<Attribute> class121Attributes = new Vector<>();
		final List<Type> class121SuperTypes = new Vector<>();
		final Collection<Operation> class121Operations = new Vector<>();
		final ClassType class121 =
				RegularClassType.create(
						class121Name,
						class121Modifiers,
						class121Attributes,
						class121SuperTypes,
						class121Operations,
						new Vector<Constructor>(),
						class121IdentifierToken,
						
						new Vector<ClassType>());
		group12Elements.add(class121);
		final IdentifierToken group2IdentifierToken =
				IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group2Name = UnqualifiedName.create(group2IdentifierToken);
		final Collection<GroupElement> group2Elements = new Vector<>();
		final Group group2 = Group.create(group2Name, group2Elements, group2IdentifierToken);
		model.addGroup(group2);
		final IdentifierToken class21IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class21Name = group2Name.addName(class21IdentifierToken);
		final Collection<ClassModifier> class21Modifiers = new Vector<>();
		final Collection<Attribute> class21Attributes = new Vector<>();
		final List<Type> class21SuperTypes = new Vector<>();
		final Collection<Operation> class21Operations = new Vector<>();
		final ClassType class21 =
				RegularClassType.create(
						class21Name,
						class21Modifiers,
						class21Attributes,
						class21SuperTypes,
						class21Operations,
						new Vector<Constructor>(),
						class21IdentifierToken,
						
						new Vector<ClassType>());
		group2Elements.add(class21);
		final IdentifierToken group21IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group21Name = group2Name.addName(group21IdentifierToken);
		final Collection<GroupElement> group21Elements = new Vector<>();
		final Group group21 = Group.create(group21Name, group21Elements, group21IdentifierToken);
		group2Elements.add(group21);
		final IdentifierToken class211IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class211Name = group21Name.addName(class211IdentifierToken);
		final Collection<ClassModifier> class211Modifiers = new Vector<>();
		final Collection<Attribute> class211Attributes = new Vector<>();
		final List<Type> class211SuperTypes = new Vector<>();
		final Collection<Operation> class211Operations = new Vector<>();
		final ClassType class211 =
				RegularClassType.create(
						class211Name,
						class211Modifiers,
						class211Attributes,
						class211SuperTypes,
						class211Operations,
						new Vector<Constructor>(),
						class211IdentifierToken,
						
						new Vector<ClassType>());
		group21Elements.add(class211);
		final ByReferenceState class21SuperTypeReferenceState =
				ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class21SuperTypeReference =
				TypeProxy.create(class211IdentifierToken, class21SuperTypeReferenceState);
		class21SuperTypes.add(class21SuperTypeReference);
		final ByReferenceState class211SuperTypeState = ByReferenceState.create(class21, class21.getTypeName());
		final TypeProxy class211Supertype = TypeProxy.create(class21IdentifierToken, class211SuperTypeState);
		class211SuperTypes.add(class211Supertype);
		final ByReferenceState class211AttributeTypeReferenceState =
				ByReferenceState.create(class121, class121.getTypeName());
		final TypeProxy class211AttributeTypeReference =
				TypeProxy.create(class121IdentifierToken, class211AttributeTypeReferenceState);
		final IdentifierToken class211AttributeTypeIdentifier =
				IdentifierToken.create("", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken class211AttributeIdentifier =
				IdentifierToken.create("classa", Position.create("", 0 + 1, 0 + 1, 0));
		final Attribute class211Attribute =
				Attribute.create(
						"classa",
						class211AttributeTypeReference,
						new Vector<AttributeModifier>(),
						class211AttributeIdentifier);
		class211Attributes.add(class211Attribute);
		final IdentifierToken group211IdentifierToken =
				IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name group211Name = group21Name.addName(group211IdentifierToken);
		final Collection<GroupElement> group211Elements = new Vector<>();
		final Group group211 = Group.create(group211Name, group211Elements, group211IdentifierToken);
		group21Elements.add(group211);
		final IdentifierToken class2111IdentifierToken =
				IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final Name class2111Name = group211Name.addName(class2111IdentifierToken);
		final Collection<ClassModifier> class2111Modifiers = new Vector<>();
		final Collection<Attribute> class2111Attributes = new Vector<>();
		final List<Type> class2111SuperTypes = new Vector<>();
		final ClassType class2111 =
				RegularClassType.create(
						class2111Name,
						class2111Modifiers,
						class2111Attributes,
						class2111SuperTypes,
						class211Operations,
						new Vector<Constructor>(),
						class2111IdentifierToken,
						
						new Vector<ClassType>());
		group211Elements.add(class2111);
		final ByReferenceState class2111SuperTypeState = ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class2111Supertype = TypeProxy.create(class211IdentifierToken, class2111SuperTypeState);
		class2111SuperTypes.add(class2111Supertype);
		final ByReferenceState class2111AttributeTypeReferenceState =
				ByReferenceState.create(class211, class211.getTypeName());
		final TypeProxy class2111AttributeTypeReference =
				TypeProxy.create(class211AttributeTypeIdentifier, class2111AttributeTypeReferenceState);
		final IdentifierToken class2111AttributeIdentifier =
				IdentifierToken.create("class21", Position.create("", 0 + 1, 0 + 1, 0));
		final Attribute class2111Attribute =
				Attribute.create(
						"class21",
						class2111AttributeTypeReference,
						new Vector<AttributeModifier>(),
						class2111AttributeIdentifier);
		class2111Attributes.add(class2111Attribute);
		
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		
		final Model result = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(result);
		assertEquals(1, v.getFailResult().size());
		assertEquals(1, v.getTypeReferencer().getExceptions().size());
		assertEquals(0, v.getOkResult().size());
	}
	
	/**
	 * Test the referencing of: group1:group = [class1:class= {} abstract; class2 : class = group1>class1 + {};] ;.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSimpleReferencingFullQualified() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {} abstract; class2 : class = group1>class1 + {};] ; ");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of:group1 : group = [ class1 : class = {} abstract; class2 : class = class1 + {};] ;.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSimpleReferencingUnqualified() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {} abstract; class2 : class = class1 + {};] ; ");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of:group1 : group = [ class1 : class = {} abstract; class2 : class = {farbe : Integer;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSimpleReferencingBaseTypeAttribute() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {} abstract; class2 : class = {farbe : Integer;};] ; ");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of:group1:group=[class1:class={}abstract;
	 * group2:group=[class1:class={farben:String*;alter:Integer;};
	 * class2:class=class1+{x:class2;};class3:class=group1>class1+{};];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSameClassNameDifferentGroups() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1:group=[class1:class={}abstract;group2:group=[class1:class={farben:String*;alter:Integer;};"
								+ "class2:class=class1+{x:class2;};class3:class=group1>class1+{};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of:group1:group=[class1:class={attr1:Class2 prior;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testNoClassException1() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class={" + "attr1:Class2 prior;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(0, v.getOkResult().size());
		assertEquals(1, v.getTypeReferencer().getExceptions().size());
	}
	
	/**
	 * Test the referencing of: group1:group=[class1:class={attr1:group2>Class2 prior;};class2:class={};];
	 * group2:group=[class3:class={};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testNoClassException2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class={" + "attr1:group2>Class2 prior;};"
						+ "class2:class={};];group2:group=[" + "class3:class={};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(0, v.getOkResult().size());
		assertEquals(1, v.getTypeReferencer().getExceptions().size());
	}
	
	/**
	 * Test the referencing of: group1:group=[class1:class={};
	 * group2:group=[class2:class={};group3:group=[class3:class={a:group2>class2;};];];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testRelativeQualifiers() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class={};"
						+ "group2:group=[class2:class={};group3:group=[class3:class={a:group2>class2;};];];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of: group1:group=[class1:class={a:class1;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testClassAsAttribute() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("group1:group=[class1:class={a:class1;};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of: group1:group=[String:class={}; group2:group=[class1:class={a:String;};];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testUserString() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[String:class={};" + "group2:group=[class1:class={a:String;};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
		this.assertOperations.assertAllReferencesByType(model);
	}
	
	/**
	 * Test the referencing of: group1:group=[class1:class={};group1:group=[class2:class=group1>class1+{};];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testFindNoClassBecauseLocal() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[class1:class={};group1:group=["
						+ "class2:class=group1>class1+{};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(0, v.getOkResult().size());
		assertEquals(1, v.getTypeReferencer().getExceptions().size());
	}
	
	/**
	 * Tests the elimination of duplicate Sums with: outerGroup:group=[outerClass:class={ op:[[(sum:{String, Integer})
	 * -> Integer]];};innerGroup:group=[innerClass:class={ duplicateSumOp:[[(sameSum:{Integer, String} ) ->
	 * Integer]];};];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testDuplicateSumElimination() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("outerGroup:group=[outerClass:class={"
						+ "op:[[(sum:{String, Integer}) -> Integer]];};innerGroup:group=[innerClass:class={"
						+ "duplicateSumOp:[[(sameSum:{Integer, String} ) -> Integer]];};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		// final Parser parser = Parser.create(output);
		// final Model model = parser.parse();
		// final ReferencerTestReturnValue v = new ReferencerTestReturnValue(model);
		
		// TODO: Warum gibt es hier kein assert?
		// final int expectedDifferentReferences = 6;
		// this.assertOperations.assertNumberOfDifferentReferences(model, 6);
	}
	
	/**
	 * Tests the elimination of duplicate Sums with: outerGroup:group=[outerClass:class={ op:[[(param:Integer) ->
	 * (x:Integer, y:Integer)]];};innerGroup:group=[ innerClass:class={duplicateSumOp:[[(param:Integer) -> (y:Integer,
	 * x:Integer)]];};];];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testDuplicateProductElimination() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("outerGroup:group=[outerClass:class={"
						+ "op:[[(param:Integer) -> (x:Integer, y:Integer)]];};innerGroup:group=["
						+ "innerClass:class={duplicateSumOp:[[(param:Integer) -> (y:Integer, x:Integer)]];};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		// final Parser parser = Parser.create(output);
		// final Model model = parser.parse();
		// final ReferencerTestReturnValue v = new ReferencerTestReturnValue(model);
		// TODO: Warum ist das auskommentiert?
		// int expectedDifferentReferences = 6;
		// this.assertOperations.assertNumberOfDifferentReferences(model, 6);
	}
	
	/**
	 * Tests the building of Anything with: group1:group=[class1:class={attribute:Anything;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSimpleAnything1() throws Exception {
		
		final IdentifierToken class1Identifier = IdentifierToken.create("class1", Position.create("", 1, 15, 14));
		final IdentifierToken group1Identifier = IdentifierToken.create("group1", Position.create("", 1, 15, 14));
		
		final SimpleScannerInput input = new SimpleScannerInput("group1:group=[class1:class={attribute:Anything;};];");
		
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		assertEquals(1, v.getOkResult().size());
		assertEquals(0, v.getFailResult().size());
		this.assertOperations.assertAllReferencesByType(model);
		
		final Vector<Attribute> attributes = new Vector<>();
		
		final QualifiedName name =
				QualifiedName
						.create(UnqualifiedName.create(group1Identifier), UnqualifiedName.create(class1Identifier));
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						DummyToken.getInstance(),
						
						new Vector<ClassType>());
		ByReferenceState classState = ByReferenceState.create(clazz, clazz.getTypeName());
		TypeProxy proxy = TypeProxy.create(class1Identifier, classState);
		final SumType sum = SumType.create(DummyToken.getInstance());
		sum.add(proxy);
		
		final BaseType string = model.getString();
		classState = ByReferenceState.create(string, string.getTypeName());
		proxy = TypeProxy.create(DummyToken.getInstance(), classState);
		sum.add(proxy);
		
		final BaseType integer = model.getInteger();
		classState = ByReferenceState.create(integer, integer.getTypeName());
		proxy = TypeProxy.create(DummyToken.getInstance(), classState);
		sum.add(proxy);
		
		final Group g = model.getGroups().iterator().next();
		final ClassType c = (ClassType) g.getGroupElements().iterator().next();
		final Attribute a = c.getAttributes().iterator().next();
		final TypeProxy t = (TypeProxy) a.getAttrType();
		final ByReferenceState state = (ByReferenceState) t.getState();
		assertEquals(sum, state.getTarget());
		
	}
	
	/**
	 * Tests the building of Anything with: group1:group=[class1:class={attribute:(irgendwas:Anything);};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSimpleAnything3() throws Exception {
		
		final IdentifierToken class1Identifier = IdentifierToken.create("class1", Position.create("", 1, 15, 14));
		final IdentifierToken group1Identifier = IdentifierToken.create("group1", Position.create("", 1, 15, 14));
		final IdentifierToken anythingIdentifier = IdentifierToken.create("Anything", DummyToken.getDummyPosition());
		final IdentifierToken irgendwasIdentifier = IdentifierToken.create("irgendwas", DummyToken.getDummyPosition());
		
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=[" + "class1:class={attribute:(irgendwas:Anything);};];");
		
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		assertEquals(1, v.getOkResult().size());
		assertEquals(0, v.getFailResult().size());
		this.assertOperations.assertAllReferencesByType(model);
		
		final Vector<Attribute> attributes = new Vector<>();
		
		final QualifiedName name =
				QualifiedName
						.create(UnqualifiedName.create(group1Identifier), UnqualifiedName.create(class1Identifier));
		
		final SumType sum = SumType.create(DummyToken.getInstance());
		final ByReferenceState sumState = ByReferenceState.create(sum, UnqualifiedName.create(anythingIdentifier));
		final TypeProxy sumProxy = TypeProxy.create(anythingIdentifier, sumState);
		
		final ProductElementType element = ProductElementType.create("irgendwas", sumProxy, irgendwasIdentifier);
		final ProductType product = ProductType.create(DummyToken.getInstance());
		product.addElement(element);
		
		sum.add(product);
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						DummyToken.getInstance(),
						
						new Vector<ClassType>());
		ByReferenceState classState = ByReferenceState.create(clazz, clazz.getTypeName());
		TypeProxy proxy = TypeProxy.create(class1Identifier, classState);
		
		sum.add(proxy);
		
		final BaseType string = model.getString();
		classState = ByReferenceState.create(string, string.getTypeName());
		proxy = TypeProxy.create(DummyToken.getInstance(), classState);
		sum.add(proxy);
		
		final BaseType integer = model.getInteger();
		classState = ByReferenceState.create(integer, integer.getTypeName());
		proxy = TypeProxy.create(DummyToken.getInstance(), classState);
		sum.add(proxy);
		
		final Group g = model.getGroups().iterator().next();
		final ClassType c = (ClassType) g.getGroupElements().iterator().next();
		final Attribute a = c.getAttributes().iterator().next();
		final ProductType p = (ProductType) a.getAttrType();
		final TypeProxy s = (TypeProxy) p.getElements().get(0).getType();
		final ByReferenceState r = (ByReferenceState) s.getState();
		assertEquals(sum, r.getTarget());
	}
	
	/**
	 * Tests the building of Anything with: group1:group=[class1:class={operation:[[(param1:Anything) ->
	 * Anything]];};class2:class={attribute:Anything;};];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSimpleAnything2() throws Exception {
		
		final IdentifierToken class1Identifier = IdentifierToken.create("class1", Position.createDummyPosition());
		final IdentifierToken class2Identifier = IdentifierToken.create("class2", Position.createDummyPosition());
		final IdentifierToken group1Identifier = IdentifierToken.create("group1", Position.createDummyPosition());
		final IdentifierToken anythingIdentifier = IdentifierToken.create("Anything", DummyToken.getDummyPosition());
		
		final SimpleScannerInput input =
				new SimpleScannerInput("group1:group=["
						+ "class1:class={operation:[[(param1:Anything) -> Anything]];};"
						+ "class2:class={attribute:Anything;};];");
		
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TypeReferencerTestReturnValue v = new TypeReferencerTestReturnValue(model);
		assertEquals(1, v.getOkResult().size());
		assertEquals(0, v.getFailResult().size());
		this.assertOperations.assertAllReferencesByType(model);
		
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		
		final QualifiedName class1Name =
				QualifiedName
						.create(UnqualifiedName.create(group1Identifier), UnqualifiedName.create(class1Identifier));
		final QualifiedName class2Name =
				QualifiedName
						.create(UnqualifiedName.create(group1Identifier), UnqualifiedName.create(class2Identifier));
		
		final ClassType clazz1 =
				RegularClassType.create(
						class1Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						DummyToken.getInstance(),
						
						new Vector<ClassType>());
		final ByReferenceState class1State = ByReferenceState.create(clazz1, clazz1.getTypeName());
		final TypeProxy proxy1 = TypeProxy.create(class1Identifier, class1State);
		
		final ClassType clazz2 =
				RegularClassType.create(
						class2Name,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						DummyToken.getInstance(),
						
						new Vector<ClassType>());
		final ByReferenceState class2State = ByReferenceState.create(clazz2, clazz2.getTypeName());
		final TypeProxy proxy2 = TypeProxy.create(class1Identifier, class2State);
		final SumType sum = SumType.create(DummyToken.getInstance());
		final ByReferenceState stateSum = ByReferenceState.create(sum, UnqualifiedName.create(anythingIdentifier));
		final TypeProxy typeProxySum = TypeProxy.create(DummyToken.getInstance(), stateSum);
		
		final ProductType product = ProductType.create(DummyToken.getInstance());
		final ProductElementType element = ProductElementType.create("param1", typeProxySum, DummyToken.getInstance());
		product.addElement(element);
		
		sum.add(product);
		
		sum.add(proxy1);
		sum.add(proxy2);
		
		final BaseType string = model.getString();
		ByReferenceState classState = ByReferenceState.create(string, string.getTypeName());
		TypeProxy proxy = TypeProxy.create(DummyToken.getInstance(), classState);
		sum.add(proxy);
		
		final BaseType integer = model.getInteger();
		classState = ByReferenceState.create(integer, integer.getTypeName());
		proxy = TypeProxy.create(DummyToken.getInstance(), classState);
		sum.add(proxy);
		
		final Group g = model.getGroups().iterator().next();
		final Iterator<GroupElement> classIterator = g.getGroupElements().iterator();
		final ClassType c1 = (ClassType) classIterator.next();
		final ClassType c2 = (ClassType) classIterator.next();
		final Operation o = c1.getOperations().iterator().next();
		
		final TypeProxy typeProxyParam = (TypeProxy) o.getParameters().getElements().get(0).getType();
		final ByReferenceState stateParam = (ByReferenceState) typeProxyParam.getState();
		assertEquals(sum, stateParam.getTarget());
		
		final TypeProxy typeProxyReferencerTestReturnValue = (TypeProxy) o.getReturnType();
		final ByReferenceState stateReferencerTestReturnValue =
				(ByReferenceState) typeProxyReferencerTestReturnValue.getState();
		assertEquals(sum, stateReferencerTestReturnValue.getTarget());
		
		final Attribute a = c2.getAttributes().iterator().next();
		final TypeProxy t = (TypeProxy) a.getAttrType();
		final ByReferenceState state = (ByReferenceState) t.getState();
		assertEquals(state.getTarget(), sum);
	}
}
